/*
 * Copyright (c) 2008-2012 GIANTS Software GmbH, Confidential, All Rights Reserved.
 * Copyright (c) 2003-2012 Christian Ammann and Stefan Geiger, Confidential, All Rights Reserved.
 */

precision mediump float;

#if defined(ENV_MAPPING)
#define LIGHT_SPACE_WORLD_SPACE
#endif

#if !defined(NORMAL_MAPPING)
#define VERTEX_LIGHTING
#endif

uniform sampler2D s_TexUnit0;
uniform sampler2D s_TexUnit1;
uniform samplerCube s_TexUnit2; //!!!
uniform vec3 u_sLightDiffuse;
uniform vec2 u_sLightConeParams;
#if defined(LIGHT_SPACE_WORLD_SPACE)
uniform vec3 u_sLightDirectionWS;
#define u_sLightDirection u_sLightDirectionWS
#else
uniform vec3 u_sLightDirection;
#endif

uniform vec3 u_dLightAmbient;
uniform vec3 u_dLightDiffuse;
#if defined(LIGHT_SPACE_WORLD_SPACE)
uniform vec3 u_dLightDirectionWS;
uniform vec3 u_dLightHalfDirectionWS;
#define u_dLightDirection u_dLightDirectionWS
#define u_dLightHalfDirection u_dLightHalfDirectionWS
#else
uniform	vec3 u_dLightDirection;
uniform	vec3 u_dLightHalfDirection;
#endif

#if defined(UV_SCROLL_ANIM)
uniform vec4 u_uvScrollParameter;
#endif

#if defined(COLOR_MASK)
uniform vec4 u_colorMask;
#endif


// Varyings

varying vec2 v_TexCoord0;
#if !defined(NORMAL_MAPPING)
varying vec3 v_Normal;
#endif
varying vec3 v_sLightDir;

#if defined(VERTEX_LIGHTING)
varying vec3 v_Color;
#endif

#if defined(ENV_MAPPING)
#if defined(NORMAL_MAPPING)
varying vec3 v_ViewDir;
#else
varying vec3 v_Reflection;
#endif
#endif

#if defined(NORMAL_MAPPING)
// transforms from light space (world or view) to tangent space
varying vec3 v_TSToLS1;
varying vec3 v_TSToLS2;
varying vec3 v_TSToLS3;
// doesn't work on tegra 2:
//varying lowp mat3 v_TSToLS;
#endif

#if defined(TIRE_TRACK) || defined(PARTICLE_SYSTEM)
varying float v_TTPSData_a;
#endif
#if defined(PARTICLE_SYSTEM)
varying float v_TTPSData_x;
#endif

#if defined(UV_SCROLL_ANIM)
varying vec4 v_attribute0;
#endif

#if defined(Z_POSTPASS)
#undef Z_PREPASS
#undef ALPHA_TESTING
#endif


#if defined(Z_PREPASS)

void main()
{
    float texelAlpha = texture2D(s_TexUnit0, v_TexCoord0).a;
    if (texelAlpha<=0.5) {
        discard;
    }
    gl_FragColor = vec4(0,0,0,0);
}

#else

void main()
{
    
#if defined(UV_SCROLL_ANIM)
    float height = v_attribute0.w;

    // Vary texture coordinates for sin/cos effect
    vec2 texCoord = v_TexCoord0;
    texCoord.x += u_uvScrollParameter.x * sin(height * u_uvScrollParameter.y);
    mediump vec4 texel = texture2D(s_TexUnit0, texCoord);
    float alpha = clamp(height * 5.0, 0.0, 1.0); 
    texel.a = texel.a * alpha;
#else
    mediump vec4 texel = texture2D(s_TexUnit0, v_TexCoord0);
#endif
#if defined(ALPHA_TESTING)
    if (texel.a<=0.5) {
        discard;
    }
#endif

    // normal mapping
#if defined(NORMAL_MAPPING)
    mediump vec3 tex = texture2D(s_TexUnit1, v_TexCoord0).xyz;
    lowp vec3 normal = normalize(tex*2.0 - 1.0);

    lowp mat3 TSToLS;
    TSToLS[0] = v_TSToLS1;
    TSToLS[1] = v_TSToLS2;
    TSToLS[2] = v_TSToLS3;

    normal = TSToLS * normal;
#else
    lowp vec3 normal = v_Normal;
#endif

    // note: We scale all lighting effects by the albedo texture at the end
    lowp vec3 lighting = vec3(0,0,0);
#if defined(VERTEX_LIGHTING)
    lighting += v_Color;
#else
    lighting += u_dLightAmbient;
#endif

    // spot light diffuse
    float attn = clamp(1.0 - dot(v_sLightDir, v_sLightDir), 0.0, 1.0);
    attn *= attn;
    mediump vec3 sLightDir = normalize(v_sLightDir);
    float spotCosAngle = dot(sLightDir, u_sLightDirection);
    float spotFactor = clamp(spotCosAngle*u_sLightConeParams.x + u_sLightConeParams.y, 0.0, 1.0);
    float spotNdotL = clamp(dot(normal, sLightDir)*5.0, 0.0, 1.0);
    lowp vec3 spotPower = vec3(spotNdotL*spotFactor * attn);
    lighting += spotPower*u_sLightDiffuse;

    // directional light diffuse + specular
    float dLightScale = 0.0;
#if !defined(VERTEX_LIGHTING)
    dLightScale += max(dot(u_dLightDirection, normal), 0.0);
#endif
#if defined(SPECULAR_LIGHTING)
    mediump float dotProduct = dot(normal, u_dLightHalfDirection);
    dLightScale += pow(clamp(dotProduct, 0.0, 1.0), 80.0);
#endif
	lowp vec3 dLightScaleVec = vec3(dLightScale);
    lighting += dLightScaleVec * u_dLightDiffuse;

#if defined(TIRE_TRACK)
    texel.a *= v_TTPSData_a;
    vec3 mDirtColor = vec3( 0.4919, 0.3764, 0.2667 )*0.6; // sRGB! Same as vehicle dirt.
    texel.xyz *= mDirtColor;
#endif
#if defined(PARTICLE_SYSTEM)
    float alpha = clamp(v_TTPSData_x, 0.0, v_TTPSData_a);
    texel.a *= alpha;
#endif

#if defined(COLOR_MASK)
    lighting = mix(lighting.xyz, lighting.xyz*u_colorMask.xyz, texel.w);
#elif defined(ENV_MAPPING)
    lighting *= 0.81;
#endif

    // env map
#if defined(ENV_MAPPING)
#if defined(NORMAL_MAPPING)
    lowp vec3 reflectVec = -reflect(v_ViewDir, normal);
#else
    lowp vec3 reflectVec = v_Reflection;
#endif
    lowp vec3 cubeTex = textureCube(s_TexUnit2, reflectVec).xyz;
    float envScale = 0.3;
#if defined(COLOR_MASK)
    envScale = u_colorMask.a;
#endif
	lowp vec3 envScaleVec = vec3(envScale * u_sLightDiffuse.y);
    lighting += cubeTex * envScaleVec;
#endif

    gl_FragColor = vec4(lighting*texel.xyz,texel.a);
}
#endif
