/*
 * Copyright (c) 2008-2012 GIANTS Software GmbH, Confidential, All Rights Reserved.
 * Copyright (c) 2003-2012 Christian Ammann and Stefan Geiger, Confidential, All Rights Reserved.
 */

precision mediump float;

uniform lowp sampler2D s_TexUnit0;
uniform	lowp vec3 u_sLightDiffuse;
uniform vec2 u_sLightConeParams;
uniform	lowp vec3 u_sLightDirectionWS;

uniform	lowp vec3 u_dLightAmbient;
uniform	lowp vec3 u_dLightDiffuse;

varying mediump vec2 v_TexCoord0;
varying mediump vec3 v_sLightDir;
varying lowp vec3 v_Color;

#if defined(Z_POSTPASS)
#undef Z_PREPASS
#undef ALPHA_TESTING
#endif

#if defined(Z_PREPASS)

void main()
{
    float texelAlpha = texture2D(s_TexUnit0, v_TexCoord0).a;
    if (texelAlpha<=0.5) {
        discard;
    }
    gl_FragColor = vec4(0,0,0,0);
}

#else

void main()
{
    lowp vec4 texel = texture2D(s_TexUnit0, v_TexCoord0);
#if defined(ALPHA_TESTING)
    if (texel.a<=0.5) {
        discard;
    }
#endif
    // spot light diffuse
    float attn = clamp(1.0 - dot(v_sLightDir, v_sLightDir), 0.0, 1.0);
    attn *= attn;
    mediump vec3 sLightDir = normalize(v_sLightDir);
    float spotCosAngle = dot(sLightDir, u_sLightDirectionWS);
    float spotFactor = clamp(spotCosAngle*u_sLightConeParams.x + u_sLightConeParams.y, 0.0, 1.0);
    // assume the normal of the crops is always facing the spot light (not correct for back faces, but only visible if camera is rotated to front view)
    float spotNdotL = 1.0; //clamp(dot(vec3(0,1,0), sLightDir)*5.0, 0.0, 1.0);
    float spotPower = spotNdotL*spotFactor * attn;
    gl_FragColor = vec4((spotPower*u_sLightDiffuse + v_Color)*texel.xyz,1.0);
}

#endif
