/*
 * Copyright (c) 2008-2012 GIANTS Software GmbH, Confidential, All Rights Reserved.
 * Copyright (c) 2003-2012 Christian Ammann and Stefan Geiger, Confidential, All Rights Reserved.
 */

precision mediump float;

uniform lowp sampler2D s_TexUnit0;
uniform lowp sampler2D s_TexUnit1;
uniform lowp samplerCube s_TexUnit2;
uniform lowp sampler2D s_TexUnit3;
#if defined(COLOR_MASK)
uniform lowp vec4 u_colorMask;
#else
uniform lowp vec4 u_dirtness;
#endif

uniform lowp vec3 u_sLightDiffuse;
uniform vec2 u_sLightConeParams;
uniform lowp vec3 u_sLightDirectionWS;
#define u_sLightDirection u_sLightDirectionWS
uniform lowp vec3 u_dLightAmbient;
uniform lowp vec3 u_dLightDiffuse;
uniform lowp vec3 u_dLightDirectionWS;
uniform lowp vec3 u_dLightHalfDirectionWS;
#define u_dLightDirection u_dLightDirectionWS
#define u_dLightHalfDirection u_dLightHalfDirectionWS

varying mediump vec2 v_TexCoord0;
varying mediump vec3 v_sLightDir;
varying lowp vec3 v_TSToLS1;
varying lowp vec3 v_TSToLS2;
varying lowp vec3 v_TSToLS3;
varying lowp vec3 v_ViewDir;

// see the D3D shader for more comments

void main()
{
    mediump vec4 texel = texture2D(s_TexUnit0, v_TexCoord0);
#if defined(ALPHA_TESTING)
    if (texel.a<=0.5) {
        discard;
    }
#endif

#if defined(COLOR_MASK)
    mediump vec4 gloss = texture2D(s_TexUnit3, v_TexCoord0);
    float dirtness = u_colorMask.w;
#else
    mediump vec3 gloss = texture2D(s_TexUnit3, v_TexCoord0).xyz;
    float dirtness = u_dirtness.w;
#endif
    float gAO = gloss.y;
    float texDirt = 1.0 - gloss.z;
    float gDirt = clamp(((texDirt*1.05 - (dirtness - 0.5)) * 2.0), 0.0, 1.0);
    float maxSpecularIntensity = 0.5;
    lowp vec3 mDirtColor = vec3( 0.4919, 0.3764, 0.2667 );
    float mDirt = gDirt * (1.0 - clamp(dirtness, 0.0, 0.9));
    float specularScale = mDirt * gAO * gloss.x * maxSpecularIntensity;

    // normal mapping
    mediump vec3 tex = texture2D(s_TexUnit1, v_TexCoord0).xyz;
    lowp vec3 normal = normalize(tex*2.0 - 1.0);
    lowp mat3 TSToLS;
    TSToLS[0] = v_TSToLS1;
    TSToLS[1] = v_TSToLS2;
    TSToLS[2] = v_TSToLS3;
    normal = TSToLS * normal;

    // lighting
    lowp vec3 diffuse = u_dLightAmbient + u_dLightDiffuse * vec3(clamp(dot(u_dLightDirection, normal), 0.0, 1.0));
    float attn = clamp(1.0 - dot(v_sLightDir, v_sLightDir), 0.0, 1.0);
    attn *= attn;
    mediump vec3 sLightDir = normalize(v_sLightDir);
    float spotCosAngle = dot(sLightDir, u_sLightDirection);
    float spotFactor = clamp(spotCosAngle*u_sLightConeParams.x + u_sLightConeParams.y, 0.0, 1.0);
    float spotNdotL = clamp(dot(normal, sLightDir)*5.0, 0.0, 1.0);
    lowp vec3 spotPower = vec3(spotNdotL*spotFactor*attn);
    lowp vec3 spot = spotPower*u_sLightDiffuse;

    mediump float dotProduct = dot(normal, u_dLightHalfDirection);
    float dLightScale = pow(clamp(dotProduct, 0.0, 1.0), 80.0);
    lowp vec3 specular = vec3(dLightScale) * u_dLightDiffuse;

    // albedo
#if defined(COLOR_MASK)
    texel.xyz = mix(texel.xyz, texel.xyz * u_colorMask.xyz, gloss.w);
#endif
    texel.xyz = mix(mDirtColor, texel.xyz, gDirt);
    texel.xyz *= gAO; 

    // environment mapping
    lowp vec3 reflectVec = -reflect(v_ViewDir, normal);
    float envScale = 0.5;
    lowp vec3 cubeTex = textureCube(s_TexUnit2, reflectVec).xyz * envScale;

    lowp vec3 color = (diffuse + spot) * texel.xyz;
    color += (cubeTex.xyz+specular)*specularScale;
    gl_FragColor = vec4(color, texel.w);
}
