/*
 * Copyright (c) 2008-2012 GIANTS Software GmbH, Confidential, All Rights Reserved.
 * Copyright (c) 2003-2012 Christian Ammann and Stefan Geiger, Confidential, All Rights Reserved.
 */

precision mediump float;

#define SPECULAR_LIGHTING

#if !defined(NORMAL_MAPPING)
#define VERTEX_LIGHTING
#endif

uniform lowp sampler2D s_TexUnit0;
uniform lowp sampler2D s_TexUnit1;
uniform	lowp vec3 u_sLightDiffuse;
uniform vec2 u_sLightConeParams;
uniform	lowp vec3 u_sLightDirection;

uniform	lowp vec3 u_dLightAmbient;
uniform	lowp vec3 u_dLightDiffuse;
uniform	vec3 u_dLightDirection;
uniform	vec3 u_dLightHalfDirection;

varying mediump vec2 v_TexCoord0;
#if !defined(NORMAL_MAPPING)
varying lowp vec3 v_Normal;
#endif
varying mediump vec3 v_sLightDir;

#if defined(VERTEX_LIGHTING)
varying lowp vec3 v_Color;
#endif

#if defined(NORMAL_MAPPING)
// transforms from light space (world or view) to tangent space
varying lowp vec3 v_TSToLS1;
varying lowp vec3 v_TSToLS2;
varying lowp vec3 v_TSToLS3;
#endif

varying vec4 v_attribute0;

void main()
{
    float alpha    = 1.0;
    float alphaPow = 1.0;
    float mSpeed   = 0.5;
    float mUVscale = 2.0;

    vec2 texCoord1 =       mUVscale * v_attribute0.xy + vec2(mSpeed, 0.0);
    vec2 texCoord2 = 1.5 * mUVscale * v_attribute0.xy + vec2(mSpeed, 0.0);
    
    vec3 t1 = texture2D(s_TexUnit1, texCoord1).xyz - 0.5;
    vec3 t2 = texture2D(s_TexUnit1, texCoord2).xyz - 0.5;

    vec3 normal = normalize(t1 + t2);
    lowp mat3 TSToLS;
    TSToLS[0] = v_TSToLS1;
    TSToLS[1] = v_TSToLS2;
    TSToLS[2] = v_TSToLS3;
    
    normal = TSToLS * normal;

    vec4 mVertexColor = vec4(1.0, 1.0, 1.0, 1.0);
    
    float mText1 = texture2D(s_TexUnit0, texCoord1).x; // ( tex2D( slurryRGB, globals.gUV.rg ) ).r;
    float mText2 = texture2D(s_TexUnit0, texCoord2).x; // ( tex2D( slurryRGB, globals.gUV.ba ) ).r;
    float mResult = mText1 + mText2;
    
    mText1 = texture2D(s_TexUnit0, texCoord1).z; // ( tex2D( slurryRGB, globals.gUV.rg  ) ).b;
    mText2 = texture2D(s_TexUnit0, texCoord2).z; // ( tex2D( slurryRGB, globals.gUV.ba  ) ).b;
    float mFade1 = 2.0 * v_TexCoord0.x;
    mResult = mix( mResult, mResult + 0.5 *( mText1 + mText2 ),  mFade1 );
    mResult *= mVertexColor.x;
    
    vec2 maskTexCoord = v_TexCoord0.yx;
    float mFadeIN  = texture2D(s_TexUnit0, vec2(0.2, 0.0) + maskTexCoord).y; // tex2D( fadeMask, float2( fadeProgress.x, 0 ) + In.vs.ALBEDOMAP_TEXCOORD ).r;
    vec2 mFlipUV = maskTexCoord;
    mFlipUV.x = 1.0 - mFlipUV.x;
    float mFadeOUT = texture2D(s_TexUnit0, vec2(0.2, 0.0) + mFlipUV).y; // tex2D( fadeMask, float2( fadeProgress.y, 0 ) + mFlipUV ).r;
    mResult *= mFadeIN * mFadeOUT;

    mResult = clamp(mResult, 0.0, 1.0);
    
    alpha    = min(mResult, clamp((v_attribute0.z - 0.615)*5.0, 0.0, 1.0));
    alphaPow = mResult*mResult;

    vec3 colorA = vec3(0.41, 0.33, 0.18); // linear RGB: float3( 0.118, 0.068, 0.016 );
    vec3 colorB = vec3(0.22, 0.17, 0.10); // linear RGB: float3( 0.027, 0.016, 0.004 );
    vec3 albedo = mix( colorA, colorB, alphaPow );
    
    
    lowp vec3 lighting = vec3(0,0,0);
#if defined(VERTEX_LIGHTING)
    lighting += v_Color;
#else
    lighting += u_dLightAmbient;
#endif

    // spot light diffuse
    float attn = clamp(1.0 - dot(v_sLightDir, v_sLightDir), 0.0, 1.0);
    attn *= attn;
    mediump vec3 sLightDir = normalize(v_sLightDir);
    float spotCosAngle = dot(sLightDir, u_sLightDirection);
    float spotFactor = clamp(spotCosAngle*u_sLightConeParams.x + u_sLightConeParams.y, 0.0, 1.0);
    float spotNdotL = clamp(dot(normal, sLightDir)*5.0, 0.0, 1.0);
    lowp vec3 spotPower = vec3(spotNdotL*spotFactor * attn);
    lighting += spotPower*u_sLightDiffuse;

    // directional light diffuse + specular
    float dLightScale = 0.0;
#if !defined(VERTEX_LIGHTING)
    dLightScale += max(dot(u_dLightDirection, normal), 0.0);
#endif
#if defined(SPECULAR_LIGHTING)
    mediump float dotProduct = dot(normal, u_dLightHalfDirection);
    dLightScale += pow(clamp(dotProduct, 0.0, 1.0), 80.0);
#endif
	lowp vec3 dLightScaleVec = vec3(dLightScale);
    lighting += dLightScaleVec * u_dLightDiffuse;

    gl_FragColor = vec4(lighting * albedo, alpha);
}
