/*
 * Copyright (c) 2008-2012 GIANTS Software GmbH, Confidential, All Rights Reserved.
 * Copyright (c) 2003-2012 Christian Ammann and Stefan Geiger, Confidential, All Rights Reserved.
 */

precision mediump float;

uniform lowp sampler2D s_TexUnit0;
uniform lowp sampler2D s_TexUnit1;
uniform lowp samplerCube s_TexUnit2;

uniform lowp vec3 u_sLightDiffuse;
uniform vec2 u_sLightConeParams;
uniform lowp vec3 u_sLightDirectionWS;
#define u_sLightDirection u_sLightDirectionWS
uniform lowp vec3 u_dLightAmbient;
uniform lowp vec3 u_dLightDiffuse;
uniform lowp vec3 u_dLightDirectionWS;
uniform lowp vec3 u_dLightHalfDirectionWS;
#define u_dLightDirection u_dLightDirectionWS
#define u_dLightHalfDirection u_dLightHalfDirectionWS

varying highp vec2 v_TexCoord0;
#ifndef VIVANTE_VARYING_BUGFIX
// The Vivante GC1000 has a maximum of 8 varying vectors as returned by glGetIntegerv(GL_MAX_VARYING_VECTORS).
varying highp vec2 v_TexCoord1;
varying highp vec2 v_TexCoord2;
#endif

varying mediump vec3 v_sLightDir;
varying lowp vec3 v_TSToLS1;
varying lowp vec3 v_TSToLS2;
varying lowp vec3 v_TSToLS3;
varying lowp vec3 v_ViewDir;
varying highp vec4 v_attribute0;
varying mediump vec3 v_sLightHalfDir;

void main()
{
    mediump vec3 t0 = (texture2D(s_TexUnit1, v_TexCoord0).xyz) - 0.5;
#ifdef VIVANTE_VARYING_BUGFIX
    mediump vec3 t1 = (texture2D(s_TexUnit1, v_TexCoord0 * 2.0).xyz) - 0.5;
    mediump vec3 t2 = (texture2D(s_TexUnit1, v_TexCoord0 * 4.0).xyz) - 0.5;

#else
    mediump vec3 t1 = (texture2D(s_TexUnit1, v_TexCoord1).xyz) - 0.5;
    mediump vec3 t2 = (texture2D(s_TexUnit1, v_TexCoord2).xyz) - 0.5;
#endif
    mediump vec3 t3 = (texture2D(s_TexUnit1, v_attribute0.xy).xyz) - 0.5;
    
    float waveIntensity = clamp((v_attribute0.w - 0.2)/0.8, 0.2, 1.0);
    vec3 normal = (t0.xyz + waveIntensity*(t1.xyz + t2.xyz) + (1.0 - waveIntensity) * t3.xyz);
    normal.xy *= vec2(waveIntensity, waveIntensity);
    mat3 TSToLS;
    TSToLS[0] = v_TSToLS1;
    TSToLS[1] = v_TSToLS2;
    TSToLS[2] = v_TSToLS3;
    normal = TSToLS * normal;
    normal = normalize(normal);

    float attn = clamp(1.0 - dot(v_sLightDir, v_sLightDir), 0.0, 1.0);
    attn *= attn;
    mediump vec3 sLightDir = normalize(v_sLightDir);
    float spotCosAngle = dot(sLightDir, u_sLightDirection);
    float spotFactor = clamp(spotCosAngle*u_sLightConeParams.x + u_sLightConeParams.y, 0.0, 1.0);
    float spotNdotL = max(dot(normal, sLightDir)*5.0, 0.0);
    float spotPower = spotNdotL*spotFactor*attn;
    vec3 spot = u_sLightDiffuse*(spotPower*0.2);

    float dotProduct = dot(normal, normalize(v_sLightHalfDir));
    vec3 specular = vec3(pow(clamp(dotProduct, 0.0, 1.0), 150.0)*u_dLightDiffuse.y*0.7);

    vec3 reflectVec = -reflect(v_ViewDir, normal);
    vec3 reflectedColor = textureCube(s_TexUnit2, reflectVec).xyz + specular;

    float lightAlpha = reflectedColor.y*0.6 + 0.3;
    vec3 color = mix(vec3(0.05, 0.16, 0.139), reflectedColor * u_dLightDiffuse.xyz, lightAlpha);
    float lightAlpha2 = color.y * 0.95 + 0.05;
    gl_FragColor = vec4(color + spot, lightAlpha2);
    //gl_FragColor = vec4(specular, 1.0);
}

