/*
 * Copyright (c) 2008-2012 GIANTS Software GmbH, Confidential, All Rights Reserved.
 * Copyright (c) 2003-2012 Christian Ammann and Stefan Geiger, Confidential, All Rights Reserved.
 */

precision mediump float;

#if !defined(NORMAL_MAPPING)
#define VERTEX_LIGHTING
#endif

uniform lowp sampler2D s_TexUnit0;
uniform lowp sampler2D s_TexUnit1;
uniform lowp sampler2D s_TexUnit2;

uniform lowp vec3 u_sLightDirection;
uniform lowp vec3 u_sLightDiffuse;
uniform vec2 u_sLightConeParams;

uniform	lowp vec3 u_dLightAmbient;
uniform	lowp vec3 u_dLightDiffuse;
uniform	vec3 u_dLightDirection;
uniform	vec3 u_dLightHalfDirection;

varying mediump vec2 v_TexCoord0;
varying mediump vec2 v_TexCoord1;
#if !defined(NORMAL_MAPPING)
varying lowp vec3 v_Color;
varying lowp vec3 v_Normal;
#endif
varying mediump vec3 v_sLightDir;
varying mediump vec4 v_attribute0;

#if defined(NORMAL_MAPPING)
// transforms from light space (world or view) to tangent space
varying lowp vec3 v_TSToLS1;
varying lowp vec3 v_TSToLS2;
varying lowp vec3 v_TSToLS3;
#endif

void main()
{
#if defined(NORMAL_MAPPING)
    mediump vec3 tex = texture2D(s_TexUnit1, v_TexCoord0).xyz;
    lowp vec3 normal = tex-0.5;
    normal.xy *= vec2(0.35, 0.35);
    
    lowp mat3 TSToLS;
    TSToLS[0] = v_TSToLS1;
    TSToLS[1] = v_TSToLS2;
    TSToLS[2] = v_TSToLS3;
    
    normal = TSToLS * normal;
    normal = normalize(normal);
#else
    lowp vec3 normal = v_Normal;
#endif

    lowp vec3 lighting = vec3(0,0,0);
#if defined(VERTEX_LIGHTING)
    lighting += v_Color;
#else
    lighting += u_dLightAmbient;
#endif

    // spot light diffuse
    float attn = clamp(1.0 - dot(v_sLightDir, v_sLightDir), 0.0, 1.0);
    attn *= attn;
    mediump vec3 sLightDir = normalize(v_sLightDir);
    float spotCosAngle = dot(sLightDir, u_sLightDirection);
    float spotFactor = clamp(spotCosAngle*u_sLightConeParams.x + u_sLightConeParams.y, 0.0, 1.0);
    float spotNdotL = clamp(dot(normal, sLightDir)*5.0, 0.0, 1.0);
    lowp vec3 spotPower = vec3(spotNdotL*spotFactor * attn);
    lighting += spotPower*u_sLightDiffuse;
    
    // directional light diffuse + specular
    float dLightScale = 0.0;
#if !defined(VERTEX_LIGHTING)
    dLightScale += max(dot(u_dLightDirection, normal), 0.0);
#endif
    lowp vec3 dLightScaleVec = vec3(dLightScale);
    lighting += dLightScaleVec * u_dLightDiffuse;

    lowp vec3 texel = texture2D(s_TexUnit0, v_TexCoord0).xyz;
    vec4 finalColor = vec4(texel * lighting * texture2D(s_TexUnit2, v_TexCoord1).x, 1.0);
    float fogFactor = pow(clamp((-v_attribute0.x - 1.6)/2.5, 0.0, 1.0), 0.5);
    gl_FragColor = mix(finalColor, vec4(0.05, 0.16, 0.139, 1.0), fogFactor);
}
