/*
 * Copyright (c) 2008-2012 GIANTS Software GmbH, Confidential, All Rights Reserved.
 * Copyright (c) 2003-2012 Christian Ammann and Stefan Geiger, Confidential, All Rights Reserved.
 */

precision highp float;
precision mediump int;

uniform mat4 u_mwMatrix;
uniform mat4 u_mvpMatrix;
uniform mediump vec4 u_sLightPositionWS;
uniform mediump vec3 u_dLightDirectionWS;
#define u_sLightPosition u_sLightPositionWS
#define u_dLightDirection u_dLightDirectionWS
uniform vec3 u_cameraPosWS;
#if defined(UV_SCROLL)
uniform vec4 u_uvOffset;
#endif

attribute vec3 Position;
attribute vec4 Normal;
attribute vec4 Tangent;
attribute vec2 TexCoord0;

varying mediump vec2 v_TexCoord0;
varying mediump vec3 v_sLightDir;
varying lowp vec3 v_ViewDir;
varying lowp vec3 v_TSToLS1;
varying lowp vec3 v_TSToLS2;
varying lowp vec3 v_TSToLS3;

#define u_MSToLSMatrix u_mwMatrix

void main()
{
    vec4 pos = vec4(Position, 1.0);
    gl_Position = u_mvpMatrix * pos;
    vec3 posLS = (u_MSToLSMatrix * pos).xyz;
    v_sLightDir = (u_sLightPosition.xyz - posLS) / 50.0;
    v_TexCoord0 = TexCoord0;
#if defined(UV_SCROLL)
    v_TexCoord0 += u_uvOffset.xy;
#endif
    // use transpose of matrix by swapping multiplication (GLSL addresses rows, not columns)
    v_TSToLS1 = normalize((u_MSToLSMatrix * vec4(Tangent.xyz,0)).xyz);
    v_TSToLS2 = normalize((u_MSToLSMatrix * vec4(cross(Normal.xyz, Tangent.xyz)*Tangent.w,0)).xyz);
    v_TSToLS3 = normalize((u_MSToLSMatrix * vec4(Normal.xyz,0)).xyz);
    v_ViewDir = normalize(u_cameraPosWS - posLS);

#ifdef SWITCH_XY
    gl_Position.xy=vec2(gl_Position.y, -gl_Position.x);
#endif
}
