/*
 * Copyright (c) 2008-2012 GIANTS Software GmbH, Confidential, All Rights Reserved.
 * Copyright (c) 2003-2012 Christian Ammann and Stefan Geiger, Confidential, All Rights Reserved.
 */

precision highp float;
precision mediump int;

#if !defined(NORMAL_MAPPING)
#define VERTEX_LIGHTING
#endif

uniform mat4 u_mwMatrix;
uniform mat4 u_mvMatrix;
uniform mat4 u_mvpMatrix;
uniform lowp vec3 u_dLightAmbient;
uniform lowp vec3 u_dLightDiffuse;
uniform	mediump vec4 u_sLightPosition;
uniform mediump vec3 u_dLightDirection;

uniform vec4 u_uvOffset;

attribute vec3 Position;
attribute vec4 Normal;
#if defined(NORMAL_MAPPING)
attribute vec4 Tangent;
#endif
attribute vec2 TexCoord0;


varying mediump vec2 v_TexCoord0;
#if !defined(NORMAL_MAPPING)
varying lowp vec3 v_Normal;
#endif
varying mediump vec3 v_sLightDir;

#if defined(VERTEX_LIGHTING)
varying lowp vec3 v_Color;
#endif

#if defined(NORMAL_MAPPING)
// transforms from light space (world or view) to tangent space
varying lowp vec3 v_TSToLS1;
varying lowp vec3 v_TSToLS2;
varying lowp vec3 v_TSToLS3;
#endif

#define u_MSToLSMatrix u_mvMatrix

varying vec4 v_attribute0;

void main()
{
    v_TexCoord0 = TexCoord0;
    vec2 uvScroll = vec2(-u_uvOffset.y*0.5, 0.0);
    v_attribute0.xy = TexCoord0 + uvScroll;
    vec4 pos=vec4(Position,1.0);
    vec4 worldPos = u_mwMatrix * pos;
    v_attribute0.zw = vec2(worldPos.y, 0.0);
    gl_Position = u_mvpMatrix * pos;
 
    vec3 posLS = (u_MSToLSMatrix * pos).xyz;
    vec3 normalLS = normalize((u_MSToLSMatrix * vec4(Normal.xyz,0.0)).xyz);
#if !defined(NORMAL_MAPPING)
	v_Normal = normalLS;
#endif
    v_sLightDir = (u_sLightPosition.xyz - posLS) / 50.0;
    
#if defined(NORMAL_MAPPING)
    // use transpose of matrix by swapping multiplication (GLSL addresses rows, not columns)
    v_TSToLS1 = normalize((u_MSToLSMatrix * vec4(Tangent.xyz,0)).xyz);
    v_TSToLS2 = normalize((u_MSToLSMatrix * vec4(cross(Normal.xyz, Tangent.xyz)*Tangent.w,0)).xyz);
    v_TSToLS3 = normalize((u_MSToLSMatrix * vec4(Normal.xyz,0)).xyz);
#endif

#if defined(VERTEX_LIGHTING)
    // directional light diffuse + ambient
    float ndotl = max(dot(normalLS, u_dLightDirection), 0.0);
    v_Color = (ndotl*u_dLightDiffuse) + u_dLightAmbient;
#endif

#ifdef SWITCH_XY
    // switch coords to landscape mode for iphone
    gl_Position.xy=vec2(gl_Position.y, -gl_Position.x);
#endif
}

