/*
 * Copyright (c) 2008-2012 GIANTS Software GmbH, Confidential, All Rights Reserved.
 * Copyright (c) 2003-2012 Christian Ammann and Stefan Geiger, Confidential, All Rights Reserved.
 */

precision highp float;
precision mediump int;

#if !defined(NORMAL_MAPPING)
#define VERTEX_LIGHTING
#endif

#if defined(GROUND_TILE)
uniform vec4 u_uvOffsets;
#endif
uniform mat4 u_mwMatrix;
uniform mat4 u_mvMatrix;
uniform mat4 u_mvpMatrix;
uniform lowp vec3 u_dLightAmbient;
uniform lowp vec3 u_dLightDiffuse;
uniform	vec3 u_dLightDirection;
uniform	vec4 u_sLightPosition;

attribute vec3 Position;
attribute vec4 Normal;
attribute vec4 Tangent;
attribute vec2 TexCoord0;

varying mediump vec2 v_TexCoord0;
varying mediump vec2 v_TexCoord1;
#if !defined(NORMAL_MAPPING)
varying lowp vec3 v_Normal;
#endif
varying mediump vec3 v_sLightDir;

#if defined(VERTEX_LIGHTING)
varying lowp vec3 v_Color;
#endif
varying mediump vec4 v_attribute0;

#if defined(NORMAL_MAPPING)
// transforms from light space (world or view) to tangent space
varying lowp vec3 v_TSToLS1;
varying lowp vec3 v_TSToLS2;
varying lowp vec3 v_TSToLS3;
#endif

#define u_MSToLSMatrix u_mvMatrix

const float worldScale = 0.001;
const vec2 worldOffset = vec2(490.0, 510.0)*worldScale;

void main()
{
#if defined(GROUND_TILE) && !defined(ANDROID)
    v_TexCoord0 = TexCoord0.xy*u_uvOffsets.y + u_uvOffsets.xx;
#else
    v_TexCoord0 = TexCoord0.xy;
#endif

    vec4 pos=vec4(Position,1.0);
    vec4 worldPos = u_mwMatrix*pos;
    v_TexCoord1.xy=worldPos.xz*worldScale + worldOffset.xy;
    gl_Position = u_mvpMatrix * pos;
    v_attribute0 = vec4(worldPos.y, 0.0, 0.0, 1.0);

#if defined(NORMAL_MAPPING)
    // use transpose of matrix by swapping multiplication (GLSL addresses rows, not columns)
    v_TSToLS1 = normalize((u_MSToLSMatrix * vec4(Tangent.xyz,0)).xyz);
    v_TSToLS2 = normalize((u_MSToLSMatrix * vec4(cross(Normal.xyz, Tangent.xyz)*Tangent.w,0)).xyz);
    v_TSToLS3 = normalize((u_MSToLSMatrix * vec4(Normal.xyz,0)).xyz);
#endif

    vec3 posLS = (u_MSToLSMatrix * pos).xyz;
    vec3 normalLS = normalize((u_MSToLSMatrix * vec4(Normal.xyz,0.0)).xyz);
#if !defined(NORMAL_MAPPING)
    v_Normal = normalLS;
#endif
    
#if defined(VERTEX_LIGHTING)
    // diffuse lighting
    float diff = max( dot(normalLS,u_dLightDirection), 0.0);
    v_Color = (diff*u_dLightDiffuse) + u_dLightAmbient;
#endif
    v_sLightDir = (u_sLightPosition.xyz - posLS) / 50.0;

    // switch coords to landscape mode for iphone
#ifdef SWITCH_XY
    gl_Position.xy=vec2(gl_Position.y, -gl_Position.x);
#endif
}
